//*  Dropper Mutator (c) 2010 MrLoathsome aka Scott Armitage *//
//*  Spawns Actors when players or monsters are killed.      *//
class DeathDrop expands Mutator config(Dropper);

var bool bInit;
var() config int NumItems;
var() config int NumDrops;
var() config int SPNumItems;
var() config int SPNumDrops;
var() config bool bRandMode;
var() config bool bSPRandMode;
var() config bool bFRotate;
var() config bool bSuicideDrop;
var() config bool bDebugMode;
var() config float DropFreq;
var() config float SPDropFreq;
var() config string DropList[255];
var() config float DLFreq[255];
var() config string SPDropList[255];
var() config float SPDLFreq[255];

function PostBeginPlay()
{
	if (bInit) return; else bInit = True;
	Level.Game.RegisterDamageMutator(Self);
	Super.PostBeginPlay();
}

function bool PreventDeath(Pawn Killed, Pawn Killer, name damageType, vector HitLocation)
{
	if (((Killed != None) && (Killed == Killer) && (!bSuicideDrop)) || (Killed == None))
	{
		if (( NextMutator != None ) && (Killed != None))
			return NextMutator.PreventDeath(Killed,Killer, damageType,HitLocation);
		return false;
	}
	if ((DropFreq != 0.0) && (ScriptedPawn(Killed) == None) && (FRand() <= DropFreq))
	{
		DropRandItems(Killed);
	} else
	if ((SPDropFreq != 0.0) && (ScriptedPawn(Killed) != None) && (FRand() <= SPDropFreq))
	{
		DropRandItemsSP(Killed);
	}
	if ( NextMutator != None )
		return NextMutator.PreventDeath(Killed,Killer, damageType,HitLocation);
	return false;
}

function DropRandItems(Pawn Victim)
{
	local int rv, i, retrycnt;
	local Actor A;
	local class<Actor> aClass;

	retrycnt = 0;
	for (i=0; i<NumDrops; i++)
		{
			if (bRandMode) { rv = (FRand()*NumItems); } else { rv = i; }
			if (FRand() <= DLFreq[rv])
			{
				aClass = class<Actor>(DynamicLoadObject(DropList[rv], class'Class'));
				if ( aClass != None )
				{
					A = Spawn(aClass,,,Victim.Location+Victim.CollisionRadius*(VRand()*(i+retrycnt+2)));
					if (A != None)
					{
						retrycnt = 0;
						A.RemoteRole = ROLE_SimulatedProxy;
						A.SetPhysics(PHYS_Falling);
						A.bCollideWorld = true;
						A.Velocity = Victim.Velocity + VRand() * 280;
						if (inventory(A) != None)
						{
							inventory(A).RespawnTime = 0.0;
							inventory(A).BecomePickup();
							if (bFRotate) { A.bBounce=True; } else { A.bBounce=False; }
							inventory(A).GotoState('PickUp', 'Dropped');
						}
						if (weapon(A) != None) weapon(A).bWeaponStay = False;
						if (bDebugMode) log("Spawned Actor = "$A);
					}
					else
					{
						retrycnt++;
						if (bDebugMode) log("Bork !  Attempt #"$retrycnt$" for Class: "$DropList[rv]);
						if (retrycnt < 8) i--;  // Try again...
					}
				}
			}
			else { i--; }
		}
}

function DropRandItemsSP(Pawn Victim)
{
	local int rv, i, retrycnt;
	local Actor A;
	local class<Actor> aClass;

	retrycnt = 0;
	for (i=0; i<SPNumDrops; i++)
		{
			if (bSPRandMode) { rv = (FRand()*SPNumItems); } else { rv = i; }
			if (FRand() <= SPDLFreq[rv])
			{
				aClass = class<Actor>(DynamicLoadObject(SPDropList[rv], class'Class'));
				if ( aClass != None )
				{
					A = Spawn(aClass,,,Victim.Location+Victim.CollisionRadius*(VRand()*(i+retrycnt+2)));
					if (A != None)
					{
						retrycnt = 0;
						A.RemoteRole = ROLE_SimulatedProxy;
						A.SetPhysics(PHYS_Falling);
						A.bCollideWorld = true;
						A.Velocity = Victim.Velocity + VRand() * 280;
						if (A.Velocity.z > 0 ) { A.Velocity.z = A.Velocity.z * -1; }
						if (inventory(A) != None)
						{
							inventory(A).RespawnTime = 0.0;
							inventory(A).BecomePickup();
							if (bFRotate) { A.bBounce=True; } else { A.bBounce=False; }
							inventory(A).GotoState('PickUp', 'Dropped');
						}
						if (weapon(A) != None) weapon(A).bWeaponStay = False;
						if (bDebugMode) log("Spawned Actor = "$A);
					}
					else
					{
						retrycnt++;
						if (bDebugMode) log("Bork ! (SP)  Attempt #"$retrycnt$" for Class: "$SPDropList[rv]);
						if (retrycnt < 8) i--;  // Try again...
					}
				}
			}
			else { i--; }
		}
}


defaultproperties
{
     NumItems=2
     NumDrops=1
     SPNumItems=2
     SPNumDrops=1
     bRandMode=True
     bSPRandMode=True
     bFRotate=False
     bSuicideDrop=False
     bDebugMode=True
     DropFreq=0.5000
     SPDropFreq=0.5000
     DropList(0)="Unreali.SearchLight"
     DropList(1)="UnrealShare.DispersionPistol"
     DLFreq(0)=1.0
     DLFreq(1)=1.0
     SPDLFreq(0)=1.0
     SPDLFreq(1)=1.0
     SPDropList(0)="Unreali.SearchLight"
     SPDropList(1)="UnrealShare.DispersionPistol"
}
